@@ -17,12 +17,11 @@
 
 #include "mongoc-cluster-private.h"
 #include "mongoc-server-stream-private.h"
+#include "mongoc-util-private.h"
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "server-stream"
 
-#define COALESCE(x, y) ((x == 0) ? (y) : (x))
-
 mongoc_server_stream_t *
 mongoc_server_stream_new (mongoc_topology_description_type_t topology_type,
                           mongoc_server_description_t *sd,
@@ -35,8 +34,8 @@ mongoc_server_stream_new (mongoc_topology_description_type_t topology_type,
 
    server_stream = bson_malloc (sizeof (mongoc_server_stream_t));
    server_stream->topology_type = topology_type;
-   server_stream->sd = sd;                       /* becomes owned */
-   server_stream->stream = stream;               /* merely borrowed */
+   server_stream->sd = sd;         /* becomes owned */
+   server_stream->stream = stream; /* merely borrowed */
 
    return server_stream;
 }
@@ -95,7 +94,8 @@ mongoc_server_stream_max_msg_size (mongoc_server_stream_t *server_stream)
  */
 
 int32_t
-mongoc_server_stream_max_write_batch_size (mongoc_server_stream_t *server_stream)
+mongoc_server_stream_max_write_batch_size (
+   mongoc_server_stream_t *server_stream)
 {
    return COALESCE (server_stream->sd->max_write_batch_size,
                     MONGOC_DEFAULT_WRITE_BATCH_SIZE);
