@@ -20,28 +20,17 @@
 
 #include "mongoc-counters-private.h"
 #include "mongoc-errno-private.h"
+#include "mongoc-socket-private.h"
 #include "mongoc-host-list.h"
-#include "mongoc-socket.h"
-#include "mongoc-trace.h"
+#include "mongoc-socket-private.h"
+#include "mongoc-trace-private.h"
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "socket"
 
 
-struct _mongoc_socket_t
-{
-#ifdef _WIN32
-   SOCKET sd;
-#else
-   int sd;
-#endif
-   int errno_;
-   int domain;
-};
-
-
 #define OPERATION_EXPIRED(expire_at) \
-   ((expire_at >= 0) && (expire_at < (bson_get_monotonic_time())))
+   ((expire_at >= 0) && (expire_at < (bson_get_monotonic_time ())))
 
 
 /*
@@ -105,12 +94,12 @@ _mongoc_socket_setnonblock (int sd)
 
 static bool
 #ifdef _WIN32
-_mongoc_socket_wait (SOCKET   sd,           /* IN */
+_mongoc_socket_wait (SOCKET sd, /* IN */
 #else
-_mongoc_socket_wait (int      sd,           /* IN */
+_mongoc_socket_wait (int sd, /* IN */
 #endif
-                     int      events,       /* IN */
-                     int64_t  expire_at)    /* IN */
+                     int events,        /* IN */
+                     int64_t expire_at) /* IN */
 {
 #ifdef _WIN32
    WSAPOLLFD pfd;
@@ -119,21 +108,11 @@ _mongoc_socket_wait (int      sd,           /* IN */
 #endif
    int ret;
    int timeout;
+   int64_t now;
 
    ENTRY;
 
-   bson_return_val_if_fail (events, false);
-
-   if (expire_at < 0) {
-      timeout = -1;
-   } else if (expire_at == 0) {
-      timeout = 0;
-   } else {
-      timeout = (int)((expire_at - bson_get_monotonic_time ()) / 1000L);
-      if (timeout < 0) {
-         timeout = 0;
-      }
-   }
+   BSON_ASSERT (events);
 
    pfd.fd = sd;
 #ifdef _WIN32
@@ -143,25 +122,135 @@ _mongoc_socket_wait (int      sd,           /* IN */
 #endif
    pfd.revents = 0;
 
+   now = bson_get_monotonic_time ();
+
+   for (;;) {
+      if (expire_at < 0) {
+         timeout = -1;
+      } else if (expire_at == 0) {
+         timeout = 0;
+      } else {
+         timeout = (int) ((expire_at - now) / 1000L);
+         if (timeout < 0) {
+            timeout = 0;
+         }
+      }
+
 #ifdef _WIN32
-   ret = WSAPoll (&pfd, 1, timeout);
+      ret = WSAPoll (&pfd, 1, timeout);
+      if (ret == SOCKET_ERROR) {
+         errno = WSAGetLastError ();
+         ret = -1;
+      }
+#else
+      ret = poll (&pfd, 1, timeout);
+#endif
+
+      if (ret > 0) {
+/* Something happened, so return that */
+#ifdef _WIN32
+         RETURN (0 != (pfd.revents & (events | POLLHUP | POLLERR)));
+#else
+         RETURN (0 != (pfd.revents & events));
+#endif
+      } else if (ret < 0) {
+         /* poll itself failed */
+
+         TRACE ("errno is: %d", errno);
+         if (MONGOC_ERRNO_IS_AGAIN (errno)) {
+            now = bson_get_monotonic_time ();
+
+            if (expire_at < now) {
+               RETURN (false);
+            } else {
+               continue;
+            }
+         } else {
+            /* poll failed for some non-transient reason */
+            RETURN (false);
+         }
+      } else {
+         /* poll timed out */
+         RETURN (false);
+      }
+   }
+}
+
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * _mongoc_socket_poll --
+ *
+ *       A multi-socket poll helper.
+ *
+ *       @events: in most cases should be POLLIN or POLLOUT.
+ *
+ *       @expire_at should be an absolute time at which to expire using
+ *       the monotonic clock (bson_get_monotonic_time(), which is in
+ *       microseconds). Or zero to not block at all. Or -1 to block
+ *       forever.
+ *
+ * Returns:
+ *       true if an event matched. otherwise false.
+ *       a timeout will return false.
+ *
+ * Side effects:
+ *       None.
+ *
+ *--------------------------------------------------------------------------
+ */
+
+ssize_t
+mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
+                    size_t nsds,               /* IN */
+                    int32_t timeout)           /* IN */
+{
+#ifdef _WIN32
+   WSAPOLLFD *pfds;
+#else
+   struct pollfd *pfds;
+#endif
+   int ret;
+   int i;
+
+   ENTRY;
+
+   BSON_ASSERT (sds);
+
+#ifdef _WIN32
+   pfds = (WSAPOLLFD *) bson_malloc (sizeof (*pfds) * nsds);
+#else
+   pfds = (struct pollfd *) bson_malloc (sizeof (*pfds) * nsds);
+#endif
+
+   for (i = 0; i < nsds; i++) {
+      pfds[i].fd = sds[i].socket->sd;
+#ifdef _WIN32
+      pfds[i].events = sds[i].events;
+#else
+      pfds[i].events = sds[i].events | POLLERR | POLLHUP;
+#endif
+      pfds[i].revents = 0;
+   }
+
+#ifdef _WIN32
+   ret = WSAPoll (pfds, nsds, timeout);
    if (ret == SOCKET_ERROR) {
       MONGOC_WARNING ("WSAGetLastError(): %d", WSAGetLastError ());
-      ret = false;
+      ret = -1;
    }
 #else
-   ret = poll (&pfd, 1, timeout);
+   ret = poll (pfds, nsds, timeout);
 #endif
 
-   if (ret > 0) {
-#ifdef _WIN32
-      RETURN (0 != (pfd.revents & (events | POLLHUP | POLLERR)));
-#else
-      RETURN (0 != (pfd.revents & events));
-#endif
+   for (i = 0; i < nsds; i++) {
+      sds[i].revents = pfds[i].revents;
    }
 
-   RETURN (false);
+   bson_free (pfds);
+
+   return ret;
 }
 
 
@@ -169,7 +258,7 @@ static bool
 #ifdef _WIN32
 _mongoc_socket_setnodelay (SOCKET sd) /* IN */
 #else
-_mongoc_socket_setnodelay (int sd)    /* IN */
+_mongoc_socket_setnodelay (int sd) /* IN */
 #endif
 {
 #ifdef _WIN32
@@ -182,12 +271,12 @@ _mongoc_socket_setnodelay (int sd)    /* IN */
    ENTRY;
 
    errno = 0;
-   ret = setsockopt (sd, IPPROTO_TCP, TCP_NODELAY,
-                     (char *)&optval, sizeof optval);
+   ret = setsockopt (
+      sd, IPPROTO_TCP, TCP_NODELAY, (char *) &optval, sizeof optval);
 
 #ifdef _WIN32
    if (ret == SOCKET_ERROR) {
-      MONGOC_WARNING ("WSAGetLastError(): %d", (int)WSAGetLastError ());
+      MONGOC_WARNING ("WSAGetLastError(): %d", (int) WSAGetLastError ());
    }
 #endif
 
@@ -215,6 +304,7 @@ int
 mongoc_socket_errno (mongoc_socket_t *sock) /* IN */
 {
    BSON_ASSERT (sock);
+   TRACE ("Current errno: %d", sock->errno_);
    return sock->errno_;
 }
 
@@ -243,6 +333,7 @@ _mongoc_socket_capture_errno (mongoc_socket_t *sock) /* IN */
 #else
    sock->errno_ = errno;
 #endif
+   TRACE ("setting errno: %d %s", sock->errno_, strerror (sock->errno_));
 }
 
 
@@ -266,6 +357,7 @@ _mongoc_socket_capture_errno (mongoc_socket_t *sock) /* IN */
 static bool
 _mongoc_socket_errno_is_again (mongoc_socket_t *sock) /* IN */
 {
+   TRACE ("errno is: %d", sock->errno_);
    return MONGOC_ERRNO_IS_AGAIN (sock->errno_);
 }
 
@@ -283,18 +375,44 @@ _mongoc_socket_errno_is_again (mongoc_socket_t *sock) /* IN */
  *       A newly allocated mongoc_socket_t on success.
  *
  * Side effects:
- *       None.
+ *       *port contains the client port number.
  *
  *--------------------------------------------------------------------------
  */
 
 mongoc_socket_t *
-mongoc_socket_accept (mongoc_socket_t *sock,      /* IN */
-                      int64_t          expire_at) /* IN */
+mongoc_socket_accept (mongoc_socket_t *sock, /* IN */
+                      int64_t expire_at)     /* IN */
+{
+   return mongoc_socket_accept_ex (sock, expire_at, NULL);
+}
+
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * mongoc_socket_accept_ex --
+ *
+ *       Private synonym for mongoc_socket_accept, returning client port.
+ *
+ * Returns:
+ *       NULL upon failure to accept or timeout.
+ *       A newly allocated mongoc_socket_t on success.
+ *
+ * Side effects:
+ *       *port contains the client port number.
+ *
+ *--------------------------------------------------------------------------
+ */
+
+mongoc_socket_t *
+mongoc_socket_accept_ex (mongoc_socket_t *sock, /* IN */
+                         int64_t expire_at,     /* IN */
+                         uint16_t *port)        /* OUT */
 {
    mongoc_socket_t *client;
-   struct sockaddr addr;
-   socklen_t addrlen = sizeof addr;
+   struct sockaddr_in addr = {0};
+   mongoc_socklen_t addrlen = sizeof addr;
    bool try_again = false;
    bool failed = false;
 #ifdef _WIN32
@@ -305,14 +423,13 @@ mongoc_socket_accept (mongoc_socket_t *sock,      /* IN */
 
    ENTRY;
 
-   bson_return_val_if_fail (sock, NULL);
+   BSON_ASSERT (sock);
 
 again:
    errno = 0;
-   sd = accept (sock->sd, &addr, &addrlen);
+   sd = accept (sock->sd, (struct sockaddr *) &addr, &addrlen);
 
    _mongoc_socket_capture_errno (sock);
-
 #ifdef _WIN32
    failed = (sd == INVALID_SOCKET);
 #else
@@ -336,9 +453,13 @@ again:
       RETURN (NULL);
    }
 
-   client = bson_malloc0 (sizeof *client);
+   client = (mongoc_socket_t *) bson_malloc0 (sizeof *client);
    client->sd = sd;
 
+   if (port) {
+      *port = ntohs (addr.sin_port);
+   }
+
    if (!_mongoc_socket_setnodelay (client->sd)) {
       MONGOC_WARNING ("Failed to enable TCP_NODELAY.");
    }
@@ -364,17 +485,17 @@ again:
  */
 
 int
-mongoc_socket_bind (mongoc_socket_t       *sock,    /* IN */
-                    const struct sockaddr *addr,    /* IN */
-                    socklen_t              addrlen) /* IN */
+mongoc_socket_bind (mongoc_socket_t *sock,       /* IN */
+                    const struct sockaddr *addr, /* IN */
+                    mongoc_socklen_t addrlen)    /* IN */
 {
    int ret;
 
    ENTRY;
 
-   bson_return_val_if_fail (sock, false);
-   bson_return_val_if_fail (addr, false);
-   bson_return_val_if_fail (addrlen, false);
+   BSON_ASSERT (sock);
+   BSON_ASSERT (addr);
+   BSON_ASSERT (addrlen);
 
    ret = bind (sock->sd, addr, addrlen);
 
@@ -407,36 +528,33 @@ mongoc_socket_bind (mongoc_socket_t       *sock,    /* IN */
 int
 mongoc_socket_close (mongoc_socket_t *sock) /* IN */
 {
-   int ret = 0;
-
    ENTRY;
 
-   bson_return_val_if_fail (sock, false);
+   BSON_ASSERT (sock);
 
 #ifdef _WIN32
    if (sock->sd != INVALID_SOCKET) {
       shutdown (sock->sd, SD_BOTH);
-      ret = closesocket (sock->sd);
+      if (0 == closesocket (sock->sd)) {
+         sock->sd = INVALID_SOCKET;
+      } else {
+         _mongoc_socket_capture_errno (sock);
+         RETURN (-1);
+      }
    }
+   RETURN (0);
 #else
    if (sock->sd != -1) {
       shutdown (sock->sd, SHUT_RDWR);
-      ret = close (sock->sd);
+      if (0 == close (sock->sd)) {
+         sock->sd = -1;
+      } else {
+         _mongoc_socket_capture_errno (sock);
+         RETURN (-1);
+      }
    }
+   RETURN (0);
 #endif
-
-   _mongoc_socket_capture_errno (sock);
-
-   if (ret == 0) {
-#ifdef _WIN32
-      sock->sd = INVALID_SOCKET;
-#else
-      sock->sd = -1;
-#endif
-      RETURN (0);
-   }
-
-   RETURN (-1);
 }
 
 
@@ -458,32 +576,32 @@ mongoc_socket_close (mongoc_socket_t *sock) /* IN */
  */
 
 int
-mongoc_socket_connect (mongoc_socket_t       *sock,      /* IN */
-                       const struct sockaddr *addr,      /* IN */
-                       socklen_t              addrlen,   /* IN */
-                       int64_t                expire_at) /* IN */
+mongoc_socket_connect (mongoc_socket_t *sock,       /* IN */
+                       const struct sockaddr *addr, /* IN */
+                       mongoc_socklen_t addrlen,    /* IN */
+                       int64_t expire_at)           /* IN */
 {
    bool try_again = false;
    bool failed = false;
    int ret;
    int optval;
-   socklen_t optlen = sizeof optval;
+   mongoc_socklen_t optlen = sizeof optval;
 
    ENTRY;
 
-   bson_return_val_if_fail (sock, false);
-   bson_return_val_if_fail (addr, false);
-   bson_return_val_if_fail (addrlen, false);
+   BSON_ASSERT (sock);
+   BSON_ASSERT (addr);
+   BSON_ASSERT (addrlen);
 
    ret = connect (sock->sd, addr, addrlen);
 
-   _mongoc_socket_capture_errno (sock);
-
 #ifdef _WIN32
    if (ret == SOCKET_ERROR) {
 #else
    if (ret == -1) {
 #endif
+      _mongoc_socket_capture_errno (sock);
+
       failed = true;
       try_again = _mongoc_socket_errno_is_again (sock);
    }
@@ -491,8 +609,8 @@ mongoc_socket_connect (mongoc_socket_t       *sock,      /* IN */
    if (failed && try_again) {
       if (_mongoc_socket_wait (sock->sd, POLLOUT, expire_at)) {
          optval = -1;
-         ret = getsockopt (sock->sd, SOL_SOCKET, SO_ERROR,
-                           (char *)&optval, &optlen);
+         ret = getsockopt (
+            sock->sd, SOL_SOCKET, SO_ERROR, (char *) &optval, &optlen);
          if ((ret == 0) && (optval == 0)) {
             RETURN (0);
          } else {
@@ -554,14 +672,14 @@ mongoc_socket_destroy (mongoc_socket_t *sock) /* IN */
  */
 
 int
-mongoc_socket_listen (mongoc_socket_t *sock,    /* IN */
-                      unsigned int     backlog) /* IN */
+mongoc_socket_listen (mongoc_socket_t *sock, /* IN */
+                      unsigned int backlog)  /* IN */
 {
    int ret;
 
    ENTRY;
 
-   bson_return_val_if_fail (sock, false);
+   BSON_ASSERT (sock);
 
    if (backlog == 0) {
       backlog = 10;
@@ -622,11 +740,11 @@ mongoc_socket_new (int domain,   /* IN */
       GOTO (fail);
    }
 
-   if (!_mongoc_socket_setnodelay (sd)) {
+   if (domain != AF_UNIX && !_mongoc_socket_setnodelay (sd)) {
       MONGOC_WARNING ("Failed to enable TCP_NODELAY.");
    }
 
-   sock = bson_malloc0 (sizeof *sock);
+   sock = (mongoc_socket_t *) bson_malloc0 (sizeof *sock);
    sock->sd = sd;
    sock->domain = domain;
 
@@ -667,36 +785,34 @@ fail:
  */
 
 ssize_t
-mongoc_socket_recv (mongoc_socket_t *sock,      /* IN */
-                    void            *buf,       /* OUT */
-                    size_t           buflen,    /* IN */
-                    int              flags,     /* IN */
-                    int64_t          expire_at) /* IN */
+mongoc_socket_recv (mongoc_socket_t *sock, /* IN */
+                    void *buf,             /* OUT */
+                    size_t buflen,         /* IN */
+                    int flags,             /* IN */
+                    int64_t expire_at)     /* IN */
 {
    ssize_t ret = 0;
    bool failed = false;
-   bool try_again = false;
 
    ENTRY;
 
-   bson_return_val_if_fail (sock, -1);
-   bson_return_val_if_fail (buf, -1);
-   bson_return_val_if_fail (buflen, -1);
+   BSON_ASSERT (sock);
+   BSON_ASSERT (buf);
+   BSON_ASSERT (buflen);
 
 again:
    sock->errno_ = 0;
 #ifdef _WIN32
-   ret = recv (sock->sd, (char *)buf, (int)buflen, flags);
+   ret = recv (sock->sd, (char *) buf, (int) buflen, flags);
    failed = (ret == SOCKET_ERROR);
 #else
    ret = recv (sock->sd, buf, buflen, flags);
    failed = (ret == -1);
 #endif
-   _mongoc_socket_capture_errno (sock);
-   try_again = (failed && _mongoc_socket_errno_is_again (sock));
-
-   if (failed && try_again) {
-      if (_mongoc_socket_wait (sock->sd, POLLIN, expire_at)) {
+   if (failed) {
+      _mongoc_socket_capture_errno (sock);
+      if (_mongoc_socket_errno_is_again (sock) &&
+          _mongoc_socket_wait (sock->sd, POLLIN, expire_at)) {
          GOTO (again);
       }
    }
@@ -705,9 +821,7 @@ again:
       RETURN (-1);
    }
 
-   DUMP_BYTES (recvbuf, (uint8_t *)buf, ret);
-
-   mongoc_counter_streams_ingress_add (ret > 0 ? ret : 0);
+   mongoc_counter_streams_ingress_add (ret);
 
    RETURN (ret);
 }
@@ -730,17 +844,17 @@ again:
  */
 
 int
-mongoc_socket_setsockopt (mongoc_socket_t *sock,    /* IN */
-                          int              level,   /* IN */
-                          int              optname, /* IN */
-                          const void      *optval,  /* IN */
-                          socklen_t        optlen)  /* IN */
+mongoc_socket_setsockopt (mongoc_socket_t *sock,   /* IN */
+                          int level,               /* IN */
+                          int optname,             /* IN */
+                          const void *optval,      /* IN */
+                          mongoc_socklen_t optlen) /* IN */
 {
    int ret;
 
    ENTRY;
 
-   bson_return_val_if_fail (sock, false);
+   BSON_ASSERT (sock);
 
    ret = setsockopt (sock->sd, level, optname, optval, optlen);
 
@@ -771,18 +885,18 @@ mongoc_socket_setsockopt (mongoc_socket_t *sock,    /* IN */
  */
 
 ssize_t
-mongoc_socket_send (mongoc_socket_t *sock,      /* IN */
-                    const void      *buf,       /* IN */
-                    size_t           buflen,    /* IN */
-                    int64_t          expire_at) /* IN */
+mongoc_socket_send (mongoc_socket_t *sock, /* IN */
+                    const void *buf,       /* IN */
+                    size_t buflen,         /* IN */
+                    int64_t expire_at)     /* IN */
 {
    mongoc_iovec_t iov;
 
-   bson_return_val_if_fail (sock, -1);
-   bson_return_val_if_fail (buf, -1);
-   bson_return_val_if_fail (buflen, -1);
+   BSON_ASSERT (sock);
+   BSON_ASSERT (buf);
+   BSON_ASSERT (buflen);
 
-   iov.iov_base = (void *)buf;
+   iov.iov_base = (void *) buf;
    iov.iov_len = buflen;
 
    return mongoc_socket_sendv (sock, &iov, 1, expire_at);
@@ -795,9 +909,9 @@ mongoc_socket_send (mongoc_socket_t *sock,      /* IN */
  * _mongoc_socket_try_sendv_slow --
  *
  *       A slow variant of _mongoc_socket_try_sendv() that sends each
- *       iovec entry one by one. This can happen if we hit EMSGSIZE on
- *       with sendmsg() on various POSIX systems (such as Solaris), or
- *       on WinXP.
+ *       iovec entry one by one. This can happen if we hit EMSGSIZE
+ *       with sendmsg() on various POSIX systems or WSASend()+WSAEMSGSIZE
+ *       on Windows.
  *
  * Returns:
  *       the number of bytes sent or -1 and errno is set.
@@ -809,9 +923,9 @@ mongoc_socket_send (mongoc_socket_t *sock,      /* IN */
  */
 
 static ssize_t
-_mongoc_socket_try_sendv_slow (mongoc_socket_t *sock,   /* IN */
-                               mongoc_iovec_t  *iov,    /* IN */
-                               size_t           iovcnt) /* IN */
+_mongoc_socket_try_sendv_slow (mongoc_socket_t *sock, /* IN */
+                               mongoc_iovec_t *iov,   /* IN */
+                               size_t iovcnt)         /* IN */
 {
    ssize_t ret = 0;
    size_t i;
@@ -824,13 +938,14 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock,   /* IN */
    BSON_ASSERT (iovcnt);
 
    for (i = 0; i < iovcnt; i++) {
-      wrote = send (sock->sd, iov [i].iov_base, iov [i].iov_len, 0);
-      _mongoc_socket_capture_errno (sock);
+      wrote = send (sock->sd, iov[i].iov_base, iov[i].iov_len, 0);
 #ifdef _WIN32
       if (wrote == SOCKET_ERROR) {
 #else
       if (wrote == -1) {
 #endif
+         _mongoc_socket_capture_errno (sock);
+
          if (!_mongoc_socket_errno_is_again (sock)) {
             RETURN (-1);
          }
@@ -839,7 +954,7 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock,   /* IN */
 
       ret += wrote;
 
-      if (wrote != iov [i].iov_len) {
+      if (wrote != iov[i].iov_len) {
          RETURN (ret);
       }
    }
@@ -868,16 +983,17 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock,   /* IN */
  */
 
 static ssize_t
-_mongoc_socket_try_sendv (mongoc_socket_t *sock,   /* IN */
-                          mongoc_iovec_t  *iov,    /* IN */
-                          size_t           iovcnt) /* IN */
+_mongoc_socket_try_sendv (mongoc_socket_t *sock, /* IN */
+                          mongoc_iovec_t *iov,   /* IN */
+                          size_t iovcnt)         /* IN */
 {
 #ifdef _WIN32
    DWORD dwNumberofBytesSent = 0;
+   int ret;
 #else
    struct msghdr msg;
+   ssize_t ret;
 #endif
-   ssize_t ret = -1;
 
    ENTRY;
 
@@ -888,37 +1004,55 @@ _mongoc_socket_try_sendv (mongoc_socket_t *sock,   /* IN */
    DUMP_IOVEC (sendbuf, iov, iovcnt);
 
 #ifdef _WIN32
-   ret = WSASend (sock->sd, (LPWSABUF)iov, iovcnt, &dwNumberofBytesSent,
-                  0, NULL, NULL);
-   ret = ret ? -1 : dwNumberofBytesSent;
+   ret = WSASend (
+      sock->sd, (LPWSABUF) iov, iovcnt, &dwNumberofBytesSent, 0, NULL, NULL);
+   TRACE ("WSASend sent: %ld (out of: %ld), ret: %d",
+          dwNumberofBytesSent,
+          iov->iov_len,
+          ret);
 #else
    memset (&msg, 0, sizeof msg);
    msg.msg_iov = iov;
    msg.msg_iovlen = (int) iovcnt;
-   ret = sendmsg (sock->sd, &msg,
-# ifdef MSG_NOSIGNAL
+   ret = sendmsg (sock->sd,
+                  &msg,
+#ifdef MSG_NOSIGNAL
                   MSG_NOSIGNAL);
 #else
                   0);
-# endif
+#endif
+   TRACE ("Send %ld out of %ld bytes", ret, iov->iov_len);
 #endif
 
-   /*
-    * Check to see if we have sent an iovec too large for sendmsg to
-    * complete. If so, we need to fallback to the slow path of multiple
-    * send() commands.
-    */
+
 #ifdef _WIN32
-   if ((ret == -1) && (errno == WSAEMSGSIZE)) {
+   if (ret == SOCKET_ERROR) {
 #else
-   if ((ret == -1) && (errno == EMSGSIZE)) {
+   if (ret == -1) {
 #endif
-      _mongoc_socket_try_sendv_slow (sock, iov, iovcnt);
+      _mongoc_socket_capture_errno (sock);
+
+/*
+ * Check to see if we have sent an iovec too large for sendmsg to
+ * complete. If so, we need to fallback to the slow path of multiple
+ * send() commands.
+ */
+#ifdef _WIN32
+      if (mongoc_socket_errno (sock) == WSAEMSGSIZE) {
+#else
+      if (mongoc_socket_errno (sock) == EMSGSIZE) {
+#endif
+         RETURN (_mongoc_socket_try_sendv_slow (sock, iov, iovcnt));
+      }
+
+      RETURN (-1);
    }
 
-   _mongoc_socket_capture_errno (sock);
-
+#ifdef _WIN32
+   RETURN (dwNumberofBytesSent);
+#else
    RETURN (ret);
+#endif
 }
 
 
@@ -946,23 +1080,29 @@ _mongoc_socket_try_sendv (mongoc_socket_t *sock,   /* IN */
  */
 
 ssize_t
-mongoc_socket_sendv (mongoc_socket_t  *sock,      /* IN */
-                     mongoc_iovec_t   *iov,       /* IN */
-                     size_t            iovcnt,    /* IN */
-                     int64_t           expire_at) /* IN */
+mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
+                     mongoc_iovec_t *in_iov, /* IN */
+                     size_t iovcnt,          /* IN */
+                     int64_t expire_at)      /* IN */
 {
    ssize_t ret = 0;
    ssize_t sent;
    size_t cur = 0;
+   mongoc_iovec_t *iov;
 
    ENTRY;
 
-   bson_return_val_if_fail (sock, -1);
-   bson_return_val_if_fail (iov, -1);
-   bson_return_val_if_fail (iovcnt, -1);
+   BSON_ASSERT (sock);
+   BSON_ASSERT (in_iov);
+   BSON_ASSERT (iovcnt);
+
+   iov = bson_malloc (sizeof (*iov) * iovcnt);
+   memcpy (iov, in_iov, sizeof (*iov) * iovcnt);
 
    for (;;) {
-      sent = _mongoc_socket_try_sendv (sock, &iov [cur], iovcnt - cur);
+      sent = _mongoc_socket_try_sendv (sock, &iov[cur], iovcnt - cur);
+      TRACE (
+         "Sent %ld (of %ld) out of iovcnt=%ld", sent, iov[cur].iov_len, iovcnt);
 
       /*
        * If we failed with anything other than EAGAIN or EWOULDBLOCK,
@@ -971,7 +1111,8 @@ mongoc_socket_sendv (mongoc_socket_t  *sock,      /* IN */
        */
       if (sent == -1) {
          if (!_mongoc_socket_errno_is_again (sock)) {
-            RETURN (ret ? ret : -1);
+            ret = -1;
+            GOTO (CLEANUP);
          }
       }
 
@@ -985,8 +1126,11 @@ mongoc_socket_sendv (mongoc_socket_t  *sock,      /* IN */
          /*
           * Subtract the sent amount from what we still need to send.
           */
-         while ((cur < iovcnt) && (sent >= (ssize_t)iov [cur].iov_len)) {
-            sent -= iov [cur++].iov_len;
+         while ((cur < iovcnt) && (sent >= (ssize_t) iov[cur].iov_len)) {
+            TRACE ("still got bytes left: sent -= iov_len: %ld -= %ld",
+                   sent,
+                   iov[cur].iov_len);
+            sent -= iov[cur++].iov_len;
          }
 
          /*
@@ -994,6 +1138,7 @@ mongoc_socket_sendv (mongoc_socket_t  *sock,      /* IN */
           * sending data over the socket.
           */
          if (cur == iovcnt) {
+            TRACE ("%s", "Finished the iovecs");
             break;
          }
 
@@ -1001,49 +1146,44 @@ mongoc_socket_sendv (mongoc_socket_t  *sock,      /* IN */
           * Increment the current iovec buffer to its proper offset and adjust
           * the number of bytes to write.
           */
-         iov [cur].iov_base = ((char *)iov [cur].iov_base) + sent;
-         iov [cur].iov_len -= sent;
+         TRACE ("Seeked io_base+%ld", sent);
+         TRACE (
+            "Subtracting iov_len -= sent; %ld -= %ld", iov[cur].iov_len, sent);
+         iov[cur].iov_base = ((char *) iov[cur].iov_base) + sent;
+         iov[cur].iov_len -= sent;
+         TRACE ("iov_len remaining %ld", iov[cur].iov_len);
 
          BSON_ASSERT (iovcnt - cur);
-         BSON_ASSERT (iov [cur].iov_len);
+         BSON_ASSERT (iov[cur].iov_len);
       } else if (OPERATION_EXPIRED (expire_at)) {
-#ifdef _WIN32
-         errno = WSAETIMEDOUT;
-#else
-         errno = ETIMEDOUT;
-#endif
-         RETURN (ret ? ret : -1);
+         GOTO (CLEANUP);
       }
 
       /*
        * Block on poll() until our desired condition is met.
        */
       if (!_mongoc_socket_wait (sock->sd, POLLOUT, expire_at)) {
-         if (ret == 0){
-#ifdef _WIN32
-            errno = WSAETIMEDOUT;
-#else
-            errno = ETIMEDOUT;
-#endif
-         }
-         RETURN (ret  ? ret : -1);
+         GOTO (CLEANUP);
       }
    }
 
+CLEANUP:
+   bson_free (iov);
+
    RETURN (ret);
 }
 
 
 int
-mongoc_socket_getsockname (mongoc_socket_t *sock,    /* IN */
-                           struct sockaddr *addr,    /* OUT */
-                           socklen_t       *addrlen) /* INOUT */
+mongoc_socket_getsockname (mongoc_socket_t *sock,     /* IN */
+                           struct sockaddr *addr,     /* OUT */
+                           mongoc_socklen_t *addrlen) /* INOUT */
 {
    int ret;
 
    ENTRY;
 
-   bson_return_val_if_fail (sock, -1);
+   BSON_ASSERT (sock);
 
    ret = getsockname (sock->sd, addr, addrlen);
 
@@ -1057,13 +1197,13 @@ char *
 mongoc_socket_getnameinfo (mongoc_socket_t *sock) /* IN */
 {
    struct sockaddr addr;
-   socklen_t len = sizeof addr;
+   mongoc_socklen_t len = sizeof addr;
    char *ret;
-   char host [BSON_HOST_NAME_MAX + 1];
+   char host[BSON_HOST_NAME_MAX + 1];
 
    ENTRY;
 
-   bson_return_val_if_fail (sock, NULL);
+   BSON_ASSERT (sock);
 
    if ((0 == getpeername (sock->sd, &addr, &len)) &&
        (0 == getnameinfo (&addr, len, host, sizeof host, NULL, 0, 0))) {
@@ -1079,7 +1219,7 @@ bool
 mongoc_socket_check_closed (mongoc_socket_t *sock) /* IN */
 {
    bool closed = false;
-   char buf [1];
+   char buf[1];
    ssize_t r;
 
    if (_mongoc_socket_wait (sock->sd, POLLIN, 0)) {
@@ -1118,21 +1258,21 @@ mongoc_socket_check_closed (mongoc_socket_t *sock) /* IN */
  */
 
 void
-mongoc_socket_inet_ntop (struct addrinfo *rp,        /* IN */
-                         char            *buf,       /* INOUT */
-                         size_t           buflen)    /* IN */
+mongoc_socket_inet_ntop (struct addrinfo *rp, /* IN */
+                         char *buf,           /* INOUT */
+                         size_t buflen)       /* IN */
 {
    void *ptr;
    char tmp[256];
 
    switch (rp->ai_family) {
    case AF_INET:
-      ptr = &((struct sockaddr_in *)rp->ai_addr)->sin_addr;
+      ptr = &((struct sockaddr_in *) rp->ai_addr)->sin_addr;
       inet_ntop (rp->ai_family, ptr, tmp, sizeof (tmp));
       bson_snprintf (buf, buflen, "ipv4 %s", tmp);
       break;
    case AF_INET6:
-      ptr = &((struct sockaddr_in6 *)rp->ai_addr)->sin6_addr;
+      ptr = &((struct sockaddr_in6 *) rp->ai_addr)->sin6_addr;
       inet_ntop (rp->ai_family, ptr, tmp, sizeof (tmp));
       bson_snprintf (buf, buflen, "ipv6 %s", tmp);
       break;
